<?php

/* --------------------------------------------------------------
  PurposeDatabaseUpdater.php 2020-01-13
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2020 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

namespace Gambio\CookieConsentPanel\Services\Purposes\Repositories;

use CI_DB_query_builder;
use Gambio\CookieConsentPanel\Services\Purposes\Interfaces\PurposeDatabaseUpdaterInterface;
use Gambio\CookieConsentPanel\Services\Purposes\Interfaces\PurposeUpdateDtoInterface;

/**
 * Class PurposeDatabaseUpdater
 * @package Gambio\CookieConsentPanel\Services\Purposes\Repositories
 */
class PurposeDatabaseUpdater implements PurposeDatabaseUpdaterInterface
{
    const PURPOSE_TABLE = 'cookie_consent_panel_purposes';
    /**
     * @var CI_DB_query_builder
     */
    protected $queryBuilder;
    
    
    /**
     * PurposeDatabaseUpdater constructor.
     *
     * @param CI_DB_query_builder $queryBuilder
     */
    public function __construct(CI_DB_query_builder $queryBuilder)
    {
        $this->queryBuilder = $queryBuilder;
    }
    
    
    /**
     * @inheritDoc
     */
    public function update(PurposeUpdateDtoInterface $dto)
    {
        if (!empty($dto->alias())) {
            $this->updateOnAlias($dto);
            
            return;
        }
        $this->updateOnId($dto);
    }
    
    
    /**
     * @param PurposeUpdateDtoInterface $dto
     */
    protected function updateOnAlias(PurposeUpdateDtoInterface $dto)
    {
        foreach ($dto->names() as $languageId => $name) {
            $this->queryBuilder->update(self::PURPOSE_TABLE,
                                        [
                                            'category_id'         => $dto->categoryId(),
                                            'purpose_description' => $dto->descriptions()[$languageId],
                                            'purpose_name'        => $dto->names()[$languageId],
                                            'purpose_status'      => (int)$dto->status(),
                                        ],
                                        [
                                            'purpose_alias' => $dto->alias(),
                                            'language_id'   => $languageId
                                        ]
            );
        }
    }
    
    
    /**
     * @param PurposeUpdateDtoInterface $dto
     */
    protected function updateOnId(PurposeUpdateDtoInterface $dto)
    {
        foreach ($dto->names() as $languageId => $name) {
            $this->queryBuilder->update(self::PURPOSE_TABLE,
                                        [
                                            'category_id'         => $dto->categoryId(),
                                            'purpose_description' => $dto->description($languageId),
                                            'purpose_name'        => $dto->name($languageId),
                                            'purpose_status'      => (int)$dto->status(),
                                        ],
                                        [
                                            'purpose_id'  => $dto->id(),
                                            'language_id' => $languageId
                                        ]
            );
        }
    }
    
    
    /**
     * @inheritDoc
     */
    public function updateStatus($id, $status)
    {
        $this->queryBuilder->update(self::PURPOSE_TABLE, ['purpose_status' => $status], ['purpose_id' => $id]);
    }
}